import 'dart:async';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';

import '../../abg_utils.dart';

List<OrderData> bookings = [];
List<OrderData> baseBooking = [];
int newBookingCount = 0;
late DateTime _last;

Function()? _jobInfoListen;
setJobInfoListen(Function()? jobInfoListen){
  _jobInfoListen = jobInfoListen;
}

late BuildContext _context;
late Function() _redraw;

setRedrawAndContext(BuildContext context, Function() redraw){
  _context = context;
  _redraw = redraw;
}

//
// listen  - "providerId",
//
Future<String?> initBookings(String listen, String userEmail) async{
  User? user = FirebaseAuth.instance.currentUser;
  if (user == null)
    return "not register";

  String providerId = "";
  if (listen == "providerId")
    providerId = await getProviderId(userEmail);
  if (listen == "customerId")
    providerId = user.uid;

  newBookingCount = 0;
  var ret = await _load(listen, providerId);
  if (ret != null)
    return ret;
  if (bookings.isEmpty)
    _last = DateTime.now();
  else{
    // _start = parent.bookings[parent.bookings.length-1].time;
    _last = bookings[0].timeUtc;
  }
  baseBooking.addAll(bookings);
  dprint("booking init bookingsCount=${bookings.length}");
  return await _listen(listen, providerId);
}

Future<String?> _load(String listen, String providerId) async{
  User? user = FirebaseAuth.instance.currentUser;
  if (user == null)
    return "not register";
  try{
    var querySnapshot = await FirebaseFirestore.instance.collection("booking")
        .where(listen, isEqualTo: providerId).get(); // listen = "providerId" or
    bookings = [];
    var index = 0;
    for (var result in querySnapshot.docs) {
      var _data = OrderData.fromJson(result.id, result.data());
      if (_data.status != "temp") {
        bookings.add(_data);
        index++;
        dprint("$index _data.viewByProvider=${_data.viewByProvider} ${_data.id} ${_data.time}");
        if (!_data.viewByProvider && listen == "providerId")
          newBookingCount++;
      }
    }
    // print("--------------> Booking load size= ${querySnapshot.docs.length}");
    bookings.sort((a, b) => b.time.compareTo(a.time));
    for (var item in bookings)
      for (var item2 in item.addon)
        item2.selected = true;
    // await FirebaseFirestore.instance.collection("settings").doc("main")
    //     .set({"booking_count_unread": 0}, SetOptions(merge:true));
  }catch(ex){
    return "booking _load " + ex.toString();
  }
  return null;
}

// ignore: cancel_subscriptions
StreamSubscription? listenBookingStream;

Future<String?> _listen(String listen, String providerId) async{
  try{
    User? user = FirebaseAuth.instance.currentUser;
    if (user == null)
      return "not register";
    listenBookingStream = FirebaseFirestore.instance.collection("booking")
        .where(listen, isEqualTo: providerId).where("time", isGreaterThan: _last)
        .snapshots().listen((querySnapshot) {
      bookings = [];
      bookings.addAll(baseBooking);
      for (var result in querySnapshot.docs) {
        var _data = OrderData.fromJson(result.id, result.data());
        if (_data.status != "temp")
          bookings.add(_data);
        //dprint("listen _data.viewByProvider=${_data.viewByProvider} ${_data.id}");
        if (!_data.viewByProvider && listen == "providerId")
          newBookingCount++;
      }
      for (var item in bookings)
        for (var item2 in item.addon)
          item2.selected = true;
      bookings.sort((a, b) => b.time.compareTo(a.time));
      if (_jobInfoListen != null) {
        _jobInfoListen!();
      }
      _redraw();
    }, onError: (Object? obj){
      messageError(_context, "Configure Indexes for booking");
      dprint("Configure Indexes for booking");
    });
  }catch(ex){
    return "listenBooking " + ex.toString();
  }
  return null;
}

Future<String?> setBookingToRead(String listen) async{
  try{
    // var index = 0;

    if (listen == "viewByProvider")
      for (var item in bookings){
        // index++;
        //dprint("$index setBookingToRead item.viewByProvider=${item.viewByProvider} ${item.id} ${item.time}");
        if (!item.viewByProvider){
          await FirebaseFirestore.instance.collection("booking").doc(item.id).set({
            "viewByProvider": true,
          }, SetOptions(merge: true));
          item.viewByProvider = true;
          for (var item2 in baseBooking)
            if (item2.id == item.id)
              item2.viewByProvider = true;
        }
        newBookingCount = 0;
      }

  }catch(ex){
    return "setBookingToRead " + ex.toString();
  }
  return null;
}
