import '../../abg_utils.dart';
import 'package:cloud_firestore/cloud_firestore.dart';

class OrderData {
  String id;
  String status;
  List<StatusHistory> history;
  //
  String customerId;
  String customer;
  String customerAvatar;
  String providerId;
  String providerPhone;
  List<StringData> provider;
  String providerAvatar;
  String serviceId;
  String serviceImage;
  List<StringData> service;
  //
  List<StringData> priceName;
  double price;
  double discPrice;
  String priceUnit; // "hourly" or "fixed"
  int count;
  String couponId;
  String couponName;
  double discount;
  String discountType; // "percent" or "fixed"
  double tax;
  double taxAdmin;
  double total;
  //
  String paymentMethod;
  String comment;
  String address;
  bool anyTime;
  DateTime selectTime;
  DateTime time;   // timestamp
  DateTime timeUtc = DateTime.now();
  DateTime timeModify; // время воследней модификации (для listener)  --- "timeModify": FieldValue.serverTimestamp(),
  bool viewByAdmin;
  bool viewByProvider;
  bool rated;
  bool finished;
  List<AddonData> addon;
  bool ver2; // version history changed
  bool ver3;
  bool paymentMethodCache; // v3

  double getPrice(){
    return discPrice != 0 ? discPrice : price;
  }

  double _getTotal(){
    var _price = count*getPrice();
    var _coupon = getCoupon();
    var _addon = getAddonsTotal();
    return _price-_coupon+_addon;
  }

  double getTaxFromPrice(){
    return _getTotal()*tax/100;
  }

  double getAddonsTotal(){
    double total = 0;
    for (var item in addon)
      if (item.selected)
        total += (item.needCount*item.price);
    return total;
  }

  double getTotal(){
    return _getTotal() + getTaxFromPrice();
  }

  double getCoupon(){
    if (couponName.isEmpty)
      return 0;
    double total = getPrice()*count;
    if (discountType == "percentage")
      return total*discount/100;
    return discount;
  }

  OrderData({this.id = "", this.status = "",
    this.customerId = "", this.customer = "", this.customerAvatar = "",
    this.providerId = "", required this.provider, this.serviceId = "", required this.service,
    this.price = 0, this.discPrice = 0, this.priceUnit = "fixed", this.count = 1, this.couponId = "",
    this.couponName = "", this.discount = 0, this.discountType = "fixed",
    this.tax = 0, this.total = 0,
    this.paymentMethod = "", this.comment = "", this.address = "", required this.time, required this.timeModify,
    this.anyTime = true, required this.selectTime, this.providerAvatar = "",
    this.providerPhone = "", required this.history, required this.priceName,
    this.viewByAdmin = false, this.viewByProvider = false,
    this.taxAdmin = 0, this.finished = false, required this.addon,
    this.rated = false, this.serviceImage = "",
    this.ver2 = true,
    this.ver3 = true,
    this.paymentMethodCache = true,
  }){
    timeUtc = time;
    time = time.toLocal();
  }

  factory OrderData.createEmpty(){
    return OrderData(service: [], provider: [], time: DateTime.now(), timeModify: DateTime.now(),
        selectTime: DateTime.now(), history: [], priceName: [], addon: []);
  }

  Map<String, dynamic> toJson() => {
    'status': status,
    'customerId': customerId,
    'customer' :  customer,
    'customerAvatar' :  customerAvatar,
    'providerId' : providerId,
    'provider' : provider.map((i) => i.toJson()).toList(),
    'providerAvatar' : providerAvatar,
    'serviceId': serviceId,
    'service': service.map((i) => i.toJson()).toList(),
    'price' : price,
    'discPrice': discPrice,
    'priceName': priceName.map((i) => i.toJson()).toList(),
    'priceUnit': priceUnit,
    'count': count,
    'couponId': couponId,
    'couponName': couponName,
    'discount': discount,
    'discountType': discountType,
    'tax' : tax,
    'taxAdmin': taxAdmin,
    'total': total,
    'paymentMethod': paymentMethod,
    'comment': comment,
    'address' : address,
    'anyTime': anyTime,
    'selectTime': selectTime.millisecondsSinceEpoch,
    'time': FieldValue.serverTimestamp(),
    'timeModify': FieldValue.serverTimestamp(),
    'providerPhone': providerPhone,
    'history': history.map((i) => i.toJson()).toList(),
    'viewByAdmin' : viewByAdmin,
    'viewByProvider' : viewByProvider,
    'rated' : rated,
    'finished': finished,
    'addon': addon.map((i) => i.toJson()).toList(),
    'serviceImage' : serviceImage,
    'ver2' : ver2,
    'ver3' : ver3,
    'paymentMethodCache': paymentMethodCache,
  };

  factory OrderData.fromJson(String id, Map<String, dynamic> data){
    List<StringData> _provider = [];
    if (data['provider'] != null)
      for (var element in List.from(data['provider'])) {
        _provider.add(StringData.fromJson(element));
      }
    List<StringData> _service = [];
    if (data['service'] != null)
      for (var element in List.from(data['service'])) {
        _service.add(StringData.fromJson(element));
      }
    List<StatusHistory> _history = [];
    if (data['history'] != null)
      for (var element in List.from(data['history'])) {
        _history.add(StatusHistory.fromJson(element));
      }
    List<StringData> _priceName = [];
    if (data['priceName'] != null)
      for (var element in List.from(data['priceName'])) {
        _priceName.add(StringData.fromJson(element));
      }
    List<AddonData> _addon = [];
    if (data['addon'] != null)
      for (var element in List.from(data['addon'])) {
        _addon.add(AddonData.fromJson(element));
      }
    return OrderData(
      id: id,
      status: (data["status"] != null) ? data["status"] : "",
      customerId: (data["customerId"] != null) ? data["customerId"] : "",
      customer: (data["customer"] != null) ? data["customer"] : "",
      customerAvatar: (data["customerAvatar"] != null) ? data["customerAvatar"] : "",
      providerId: (data["providerId"] != null) ? data["providerId"] : "",
      providerPhone: (data["providerPhone"] != null) ? data["providerPhone"] : "",
      provider: _provider,
      providerAvatar: (data["providerAvatar"] != null) ? data["providerAvatar"] : "",
      serviceId: (data["serviceId"] != null) ? data["serviceId"] : "",
      service: _service,
      price: (data["price"] != null) ? toDouble(data["price"].toString()) : 0.0,
      discPrice: (data["discPrice"] != null) ? toDouble(data["discPrice"].toString()) : 0.0,
      priceUnit: (data["priceUnit"] != null) ? data["priceUnit"] : "",
      priceName: _priceName,
      count: (data["count"] != null) ? data["count"] : 1,
      couponId: (data["couponId"] != null) ? data["couponId"] : "",
      couponName: (data["couponName"] != null) ? data["couponName"] : "",
      discount: (data["discount"] != null) ? toDouble(data["discount"].toString()) : 0.0,
      discountType: (data["discountType"] != null) ? data["discountType"] : "",
      tax: (data["tax"] != null) ? toDouble(data["tax"].toString()) : 0.0,
      total: (data["total"] != null) ? toDouble(data["total"].toString()) : 0.0,
      paymentMethod: (data["paymentMethod"] != null) ? data["paymentMethod"] : "",
      comment: (data["comment"] != null) ? data["comment"] : "",
      address: (data["address"] != null) ? data["address"] : "",
      anyTime: (data["anyTime"] != null) ? data["anyTime"] : true,
      selectTime: (data["selectTime"] != null) ? DateTime.fromMillisecondsSinceEpoch(data["selectTime"]) : DateTime.now(),
      //
      time: (data["time"] != null) ? data["time"].toDate() : DateTime.now(),
      timeModify: (data["timeModify"] != null) ? data["timeModify"].toDate() : DateTime.now(),
      //
      history: _history,
      viewByAdmin: (data["viewByAdmin"] != null) ? data["viewByAdmin"] : false,
      viewByProvider: (data["viewByProvider"] != null) ? data["viewByProvider"] : false,
      taxAdmin: (data["taxAdmin"] != null) ? toDouble(data["taxAdmin"].toString()) : 0.0,
      finished: (data["finished"] != null) ? data["finished"] : false,
      rated: (data["rated"] != null) ? data["rated"] : false,
      addon: _addon,
      serviceImage: (data["serviceImage"] != null) ? data["serviceImage"] : "",
      paymentMethodCache: (data["paymentMethodCache"] != null) ? data["paymentMethodCache"] : false,
      ver2: (data["ver2"] != null) ? data["ver2"] : true,
      ver3 : (data["ver3"] != null) ? data["ver3"] : true, // on demand v2
    );
  }

  StatusHistory getHistoryDate(String id){
    for (var item in history)
      if (item.statusId == id)
        return item;
    return StatusHistory.createEmpty();
  }

}
class StatusHistory{
  String statusId = "";
  DateTime time = DateTime.now();
  bool byCustomer;
  bool byProvider;
  bool byAdmin;
  String activateUserId;

  StatusHistory({this.statusId = "", required this.time, this.byCustomer = false,
    this.byProvider = false, this.byAdmin = false, this.activateUserId = ""
  });

  Map<String, dynamic> toJson() => {
    'statusId': statusId,
    'time': time,
    'byCustomer' : byCustomer,
    'byProvider' : byProvider,
    'byAdmin' : byAdmin,
    'activateUserId' : activateUserId,
  };

  factory StatusHistory.fromJson(Map<String, dynamic> data){
    return StatusHistory(
      statusId: (data["statusId"] != null) ? data["statusId"] : "",
      time: (data["time"] != null) ? (data["time"]).toDate().toLocal() : DateTime.now(),
      byCustomer: (data["byCustomer"] != null) ? data["byCustomer"] : false,
      byProvider: (data["byProvider"] != null) ? data["byProvider"] : false,
      byAdmin: (data["byAdmin"] != null) ? data["byAdmin"] : false,
      activateUserId: (data["activateUserId"] != null) ? data["activateUserId"] : "",
    );
  }

  factory StatusHistory.createEmpty(){
    return StatusHistory(time: DateTime.now());
  }
}



/* provider v2
class OrderData {
  String id;
  String status;
  List<StatusHistory> history;
  //
  String customerId;
  String customer;
  String customerAvatar;
  String providerId;
  String serviceImage;
  String providerPhone;
  List<StringData> provider;
  String providerAvatar;
  String serviceId;
  List<StringData> service;
  //
  List<StringData> priceName;
  double price;
  double discPrice;
  String priceUnit; // "hourly" or "fixed"
  int count;
  String couponId;
  String couponName;
  double discount;
  String discountType; // "percent" or "fixed"
  double tax;
  double taxAdmin;
  double total;
  //
  String paymentMethod;
  String comment;
  String address;
  bool anyTime;
  DateTime selectTime;
  DateTime time = DateTime.now();   // timestamp
  bool viewByAdmin;
  bool viewByProvider;
  bool finished;
  List<AddonData> addon;

  OrderData({this.id = "", this.status = "",
    this.customerId = "", this.customer = "", this.customerAvatar = "",
    this.providerId = "", required this.provider, this.serviceId = "", required this.service,
    this.price = 0, this.discPrice = 0, this.priceUnit = "fixed", this.count = 1, this.couponId = "",
    this.couponName = "", this.discount = 0, this.discountType = "fixed",
    this.tax = 0, this.total = 0,
    this.paymentMethod = "", this.comment = "", this.address = "", required this.time,
    this.anyTime = true, required this.selectTime, this.providerAvatar = "",
    this.providerPhone = "", required this.history, required this.priceName,
    this.viewByAdmin = false, this.viewByProvider = false, this.finished = false,
    this.taxAdmin = 0, required this.addon, this.serviceImage = ""
  });

  factory OrderData.createEmpty(){
    return OrderData(service: [], provider: [], time: DateTime.now(),
        selectTime: DateTime.now(), history: [], priceName: [], addon: []);
  }

  Map<String, dynamic> toJson() => {
    'status': status,
    'customerId': customerId,
    'customer' :  customer,
    'customerAvatar' :  customerAvatar,
    'providerId' : providerId,
    'provider' : provider.map((i) => i.toJson()).toList(),
    'providerAvatar' : providerAvatar,
    'serviceId': serviceId,
    'service': service.map((i) => i.toJson()).toList(),
    'price' : price,
    'discPrice': discPrice,
    'priceName': priceName.map((i) => i.toJson()).toList(),
    'priceUnit': priceUnit,
    'count': count,
    'couponId': couponId,
    'couponName': couponName,
    'discount': discount,
    'discountType': discountType,
    'tax' : tax,
    'taxAdmin': taxAdmin,
    'total': total,
    'paymentMethod': paymentMethod,
    'comment': comment,
    'address' : address,
    'anyTime': anyTime,
    'selectTime': selectTime.millisecondsSinceEpoch,
    'time': FieldValue.serverTimestamp(),
    'providerPhone': providerPhone,
    'history': history.map((i) => i.toJson()).toList(),
    'viewByAdmin' : viewByAdmin,
    'viewByProvider' : viewByProvider,
    'finished': finished,
    'addon': addon.map((i) => i.toJson()).toList(),
    'serviceImage' : serviceImage,
  };

  factory OrderData.fromJson(String id, Map<String, dynamic> data){
    List<StringData> _provider = [];
    if (data['provider'] != null)
      List.from(data['provider']).forEach((element){
        _provider.add(StringData.fromJson(element));
      });
    List<StringData> _service = [];
    if (data['service'] != null)
      List.from(data['service']).forEach((element){
        _service.add(StringData.fromJson(element));
      });
    List<StatusHistory> _history = [];
    if (data['history'] != null)
      List.from(data['history']).forEach((element){
        _history.add(StatusHistory.fromJson(element));
      });
    List<StringData> _priceName = [];
    if (data['priceName'] != null)
      List.from(data['priceName']).forEach((element){
        _priceName.add(StringData.fromJson(element));
      });
    List<AddonData> _addon = [];
    if (data['addon'] != null)
      List.from(data['addon']).forEach((element){
        _addon.add(AddonData.fromJson(element));
      });
    return OrderData(
      id: id,
      status: (data["status"] != null) ? data["status"] : "",
      customerId: (data["customerId"] != null) ? data["customerId"] : "",
      customer: (data["customer"] != null) ? data["customer"] : "",
      customerAvatar: (data["customerAvatar"] != null) ? data["customerAvatar"] : "",
      providerId: (data["providerId"] != null) ? data["providerId"] : "",
      providerPhone: (data["providerPhone"] != null) ? data["providerPhone"] : "",
      provider: _provider,
      providerAvatar: (data["providerAvatar"] != null) ? data["providerAvatar"] : "",
      serviceId: (data["serviceId"] != null) ? data["serviceId"] : "",
      service: _service,
      price: (data["price"] != null) ? toDouble(data["price"].toString()) : 0,
      discPrice: (data["discPrice"] != null) ? toDouble(data["discPrice"].toString()) : 0,
      priceUnit: (data["priceUnit"] != null) ? data["priceUnit"] : "",
      priceName: _priceName,
      count: (data["count"] != null) ? data["count"] : 1,
      couponId: (data["couponId"] != null) ? data["couponId"] : "",
      couponName: (data["couponName"] != null) ? data["couponName"] : "",
      discount: (data["discount"] != null) ? toDouble(data["discount"].toString())  : 0,
      discountType: (data["discountType"] != null) ? data["discountType"] : "",
      tax: (data["tax"] != null) ? toDouble(data["tax"].toString()) : 0,
      total: (data["total"] != null) ? toDouble(data["total"].toString()) : 0,
      paymentMethod: (data["paymentMethod"] != null) ? data["paymentMethod"] : "",
      comment: (data["comment"] != null) ? data["comment"] : "",
      address: (data["address"] != null) ? data["address"] : "",
      anyTime: (data["anyTime"] != null) ? data["anyTime"] : true,
      selectTime: (data["selectTime"] != null) ? DateTime.fromMillisecondsSinceEpoch(data["selectTime"]) : DateTime.now(),
      time: (data["time"] != null) ? data["time"].toDate().toLocal() : DateTime.now(),
      history: _history,
      viewByAdmin: (data["viewByAdmin"] != null) ? data["viewByAdmin"] : false,
      viewByProvider: (data["viewByProvider"] != null) ? data["viewByProvider"] : false,
      taxAdmin: (data["taxAdmin"] != null) ? toDouble(data["taxAdmin"].toString()) : 0,
      finished: (data["finished"] != null) ? data["finished"] : false,
      addon: _addon,
      serviceImage: (data["serviceImage"] != null) ? data["serviceImage"] : "",
    );
  }

  StatusHistory getHistoryDate(String id){
    for (var item in history)
      if (item.statusId == id)
        return item;
    return StatusHistory.createEmpty();
  }

  double getPrice(){
    return discPrice != 0 ? discPrice : price;
  }

  double _getTotal(){
    var _price = count*getPrice();
    var _coupon = getCoupon();
    var _addon = getAddonsTotal();
    return _price-_coupon+_addon;
  }

  double getTaxFromPrice(){
    return _getTotal()*tax/100;
  }

  double getAddonsTotal(){
    double total = 0;
    for (var item in addon)
      if (item.selected)
        total += (item.needCount*item.price);
    return total;
  }

  double getTotal(){
    return _getTotal() + getTaxFromPrice();
  }

  double getCoupon(){
    if (couponName.isEmpty)
      return 0;
    double total = getPrice()*count;
    if (discountType == "percentage")
      return total*discount/100;
    return discount;
  }

  // double getTaxFromPrice(){
  //   return (count*getPrice()-getCoupon())*tax/100;
  // }
  //
  // double getTotal(){
  //   return count*getPrice()+getTaxFromPrice()-getCoupon();
  // }
  //
  // double getCoupon(){
  //   if (couponName.isEmpty)
  //     return 0;
  //   double total = getPrice()*count;
  //   if (discountType == "percentage")
  //     return total*discount/100;
  //   return discount;
  // }

}

class StatusHistory{
  String statusId = "";
  DateTime time = DateTime.now();
  bool byCustomer;
  bool byProvider;
  bool byAdmin;
  String activateUserId;

  StatusHistory({this.statusId = "", required this.time, this.byCustomer = false,
    this.byProvider = false, this.byAdmin = false, this.activateUserId = ""
  });

  Map<String, dynamic> toJson() => {
    'statusId': statusId,
    'time': time,
    'byCustomer' : byCustomer,
    'byProvider' : byProvider,
    'byAdmin' : byAdmin,
    'activateUserId' : activateUserId,
  };

  factory StatusHistory.fromJson(Map<String, dynamic> data){
    return StatusHistory(
      statusId: (data["statusId"] != null) ? data["statusId"] : "",
      time: (data["time"] != null) ? (data["time"]).toDate().toLocal() : DateTime.now(),
      byCustomer: (data["byCustomer"] != null) ? data["byCustomer"] : false,
      byProvider: (data["byProvider"] != null) ? data["byProvider"] : false,
      byAdmin: (data["byAdmin"] != null) ? data["byAdmin"] : false,
      activateUserId: (data["activateUserId"] != null) ? data["activateUserId"] : "",
    );
  }

  factory StatusHistory.createEmpty(){
    return StatusHistory(time: DateTime.now());
  }
}

 */