import 'package:flutter/material.dart';
import '../../abg_utils.dart';

class ProductData {
  String id;
  List<StringData> name;
  List<StringData> descTitle;
  List<StringData> desc;
  double tax;
  double taxAdmin;
  bool visible;
  List<PriceData> price;
  List<ImageData> gallery;
  Duration duration = Duration();
  List<String> category;
  List<String> providers; // Id
  //
  bool select = false;
  final dataKey = GlobalKey();
  List<String> assetsCategory;
  List<String> assetsProvider;
  List<String> assetsGallery;
  //
  int rating1;
  int rating2;
  int rating3;
  int rating4;
  int rating5;
  int count = 0;
  double rating = 0;
  int favoritesCount;
  //
  List<AddonData> addon;
  bool unavailable;

  ProductData(this.id, this.name, {this.visible = true, required this.desc,
    required this.gallery, required this.descTitle, required this.price,
    required this.duration, required this.category, required this.providers, this.tax = 0,
    this.assetsCategory = const [], this.assetsProvider = const [], this.assetsGallery = const [],
    this.rating1 = 0, this.rating2 = 0, this.rating3 = 0, this.rating4 = 0, this.rating5 = 0,
    this.count = 0, this.rating = 0, this.taxAdmin = 0, this.favoritesCount = 0, required this.addon,
    this.unavailable = false
  });

  factory ProductData.createEmpty(){
    return ProductData("", [], gallery: [], price: [], desc: [], descTitle: [],
        duration: Duration(), category: [], providers: [], addon: []);
  }

  Map<String, dynamic> toJson() => {
    'name': name.map((i) => i.toJson()).toList(),
    'tax': tax,
    'descTitle': descTitle.map((i) => i.toJson()).toList(),
    'desc': desc.map((i) => i.toJson()).toList(),
    'visible': visible,
    'price': price.map((i) => i.toJson()).toList(),
    'gallery': gallery.map((i) => i.toJson()).toList(),
    'duration': duration.inMilliseconds,
    'category' : category,
    'providers': providers,
    'rating1': rating1,
    'rating2': rating2,
    'rating3': rating3,
    'rating4': rating4,
    'rating5': rating5,
    'taxAdmin': taxAdmin,
    'favoritesCount' : favoritesCount,
    'addon': addon.map((i) => i.toJson()).toList(),
    'unavailable': unavailable,
  };

  factory ProductData.fromJson(String id, Map<String, dynamic> data){
    List<StringData> _name = [];
    if (data['name'] != null)
      for (var element in List.from(data['name'])) {
        _name.add(StringData.fromJson(element));
      }
    List<StringData> _descTitle = [];
    if (data['descTitle'] != null)
      for (var element in List.from(data['descTitle'])) {
        _descTitle.add(StringData.fromJson(element));
      }
    List<StringData> _desc = [];
    if (data['desc'] != null)
      for (var element in List.from(data['desc'])) {
        _desc.add(StringData.fromJson(element));
      }
    List<PriceData> _price = [];
    if (data['price'] != null)
      for (var element in List.from(data['price'])) {
        _price.add(PriceData.fromJson(element));
      }
    List<ImageData> _gallery = [];
    if (data['gallery'] != null)
      for (var element in List.from(data['gallery'])) {
        _gallery.add(ImageData.fromJson(element));
      }
    List<String> _category = [];
    if (data['category'] != null)
      for (var element in List.from(data['category'])) {
        _category.add(element);
      }
    List<String> _providers = [];
    if (data['providers'] != null)
      for (var element in List.from(data['providers'])) {
        _providers.add(element);
      }
    List<AddonData> _addon = [];
    if (data['addon'] != null)
      for (var element in List.from(data['addon'])) {
        _addon.add(AddonData.fromJson(element));
      }

    var rating1 = (data["rating1"] != null) ? toInt(data["rating1"].toString()) : 0;
    var rating2 = (data["rating2"] != null) ? toInt(data["rating2"].toString()) : 0;
    var rating3 = (data["rating3"] != null) ? toInt(data["rating3"].toString()) : 0;
    var rating4 = (data["rating4"] != null) ? toInt(data["rating4"].toString()) : 0;
    var rating5 = (data["rating5"] != null) ? toInt(data["rating5"].toString()) : 0;
    var _count = rating1+rating2+rating3+rating4+rating5;
    double _rating = 0;
    if (_count != 0)
      _rating = (rating1*1 + rating2*2 + rating3*3 + rating4*4 + rating5*5)/_count;

    return ProductData(
      id,
      _name,
      tax: (data["tax"] != null) ? toDouble(data["tax"].toString()) : 0.0,
      descTitle: _descTitle,
      desc: _desc,
      visible: (data["visible"] != null) ? data["visible"] : true,
      price: _price,
      gallery: _gallery,
      duration: (data["duration"] != null) ? Duration(milliseconds : data["duration"]) : Duration(),
      category: _category,
      providers: _providers,
      rating1: rating1,
      rating2: rating2,
      rating3: rating3,
      rating4: rating4,
      rating5: rating5,
      count: _count,
      rating: _rating,
      taxAdmin: (data["taxAdmin"] != null) ? toDouble(data["taxAdmin"].toString()) : 0.0,
      favoritesCount: (data["favoritesCount"] != null) ? toInt(data["favoritesCount"].toString()) : 0,
      addon: _addon,
      unavailable: (data["unavailable"] != null) ? data["unavailable"] : false,
    );
  }

  //
  //
  //
  double getMinPrice(){
    double _min = double.maxFinite;
    for (var item in price) {
      if (item.discPrice == 0) {
        if (_min > item.price)
          _min = item.price;
      }else {
        if (_min > item.discPrice)
          _min = item.discPrice;
      }
    }
    if (_min == double.maxFinite)
      return _min;
    return _min;
  }
}




/* provider v1

class ProductData {
  String id;
  List<StringData> name;
  List<StringData> descTitle;
  List<StringData> desc;
  double tax;
  bool visible;
  List<PriceData> price;
  List<ImageData> gallery;
  Duration duration = Duration();
  List<String> category;
  List<String> providers; // Id
  //
  bool select = false;
  final dataKey = new GlobalKey();
  //
  int rating1;
  int rating2;
  int rating3;
  int rating4;
  int rating5;
  int count = 0;
  double rating = 0;
  List<AddonData> addon;

  ProductData(this.id, this.name, {this.visible = true, required this.desc,
    required this.gallery, required this.descTitle, required this.price,
    required this.duration, required this.category, required this.providers, this.tax = 0,
    this.rating1 = 0, this.rating2 = 0, this.rating3 = 0, this.rating4 = 0, this.rating5 = 0,
    this.count = 0, this.rating = 0, required this.addon
  });

  factory ProductData.createEmpty(){
    return ProductData("", [], gallery: [], price: [], desc: [], descTitle: [],
        duration: Duration(), category: [], providers: [], addon: []);
  }

  Map<String, dynamic> toJson() => {
    'name': name.map((i) => i.toJson()).toList(),
    'tax': tax,
    'descTitle': descTitle.map((i) => i.toJson()).toList(),
    'desc': desc.map((i) => i.toJson()).toList(),
    'visible': visible,
    'price': price.map((i) => i.toJson()).toList(),
    'gallery': gallery.map((i) => i.toJson()).toList(),
    'duration': duration.inMilliseconds,
    'category' : category,
    'providers': providers,
    'rating1': rating1,
    'rating2': rating2,
    'rating3': rating3,
    'rating4': rating4,
    'rating5': rating5,
    'addon': addon.map((i) => i.toJson()).toList(),
  };

  factory ProductData.fromJson(String id, Map<String, dynamic> data){
    List<StringData> _name = [];
    if (data['name'] != null)
      List.from(data['name']).forEach((element){
        _name.add(StringData.fromJson(element));
      });
    List<StringData> _descTitle = [];
    if (data['descTitle'] != null)
      List.from(data['descTitle']).forEach((element){
        _descTitle.add(StringData.fromJson(element));
      });
    List<StringData> _desc = [];
    if (data['desc'] != null)
      List.from(data['desc']).forEach((element){
        _desc.add(StringData.fromJson(element));
      });
    List<PriceData> _price = [];
    if (data['price'] != null)
      List.from(data['price']).forEach((element){
        _price.add(PriceData.fromJson(element));
      });
    List<ImageData> _gallery = [];
    if (data['gallery'] != null)
      List.from(data['gallery']).forEach((element){
        _gallery.add(ImageData.fromJson(element));
      });
    List<String> _category = [];
    if (data['category'] != null)
      List.from(data['category']).forEach((element){
        _category.add(element);
      });
    List<String> _providers = [];
    if (data['providers'] != null)
      List.from(data['providers']).forEach((element){
        _providers.add(element);
      });

    var rating1 = (data["rating1"] != null) ? toInt(data["rating1"].toString()) : 0;
    var rating2 = (data["rating2"] != null) ? toInt(data["rating2"].toString()) : 0;
    var rating3 = (data["rating3"] != null) ? toInt(data["rating3"].toString()) : 0;
    var rating4 = (data["rating4"] != null) ? toInt(data["rating4"].toString()) : 0;
    var rating5 = (data["rating5"] != null) ? toInt(data["rating5"].toString()) : 0;
    var _count = rating1+rating2+rating3+rating4+rating5;
    double _rating = 0;
    if (_count != 0)
      _rating = (rating1*1 + rating2*2 + rating3*3 + rating4*4 + rating5*5)/_count;

    List<AddonData> _addon = [];
    if (data['addon'] != null)
      List.from(data['addon']).forEach((element){
        _addon.add(AddonData.fromJson(element));
      });
    return ProductData(
      id,
      _name,
      tax: (data["tax"] != null) ? toDouble(data["tax"].toString()) : 0,
      descTitle: _descTitle,
      desc: _desc,
      visible: (data["visible"] != null) ? data["visible"] : "",
      price: _price,
      gallery: _gallery,
      duration: (data["duration"] != null) ? Duration(milliseconds : data["duration"]) : Duration(),
      category: _category,
      providers: _providers,
      rating1: rating1,
      rating2: rating2,
      rating3: rating3,
      rating4: rating4,
      rating5: rating5,
      count: _count,
      rating: _rating,
      addon: _addon,
    );
  }

  //
  //
  //
  double getMinPrice(){
    double _min = double.maxFinite;
    for (var item in price) {
      if (item.discPrice == 0) {
        if (_min > item.price)
          _min = item.price;
      }else {
        if (_min > item.discPrice)
          _min = item.discPrice;
      }
    }
    if (_min == double.maxFinite)
      return 0;
    return _min;
  }
}

 */

/* site


class ServiceData {
  String id;
  List<StringData> name;
  List<StringData> descTitle;
  List<StringData> desc;
  double tax;
  double taxAdmin;
  bool visible;
  List<PriceData> price;
  List<ImageData> gallery;
  Duration duration = Duration();
  List<String> category;
  List<String> providers; // Id
  //
  bool select = false;
  final dataKey = new GlobalKey();
  //
  int rating1;
  int rating2;
  int rating3;
  int rating4;
  int rating5;
  int count = 0;
  double rating = 0;
  //
  List<AddonData> addon;

  ServiceData(this.id, this.name, {this.visible = true, required this.desc,
    required this.gallery, required this.descTitle, required this.price,
    required this.duration, required this.category, required this.providers, this.tax = 0,
    this.rating1 = 0, this.rating2 = 0, this.rating3 = 0, this.rating4 = 0, this.rating5 = 0,
    this.count = 0, this.rating = 0, this.taxAdmin = 0, required this.addon
  });

  factory ServiceData.createEmpty(){
    return ServiceData("", [], gallery: [], price: [], desc: [], descTitle: [],
        duration: Duration(), category: [], providers: [], addon: []);
  }

  Map<String, dynamic> toJson() => {
    'name': name.map((i) => i.toJson()).toList(),
    'tax': tax,
    'descTitle': descTitle.map((i) => i.toJson()).toList(),
    'desc': desc.map((i) => i.toJson()).toList(),
    'visible': visible,
    'price': price.map((i) => i.toJson()).toList(),
    'gallery': gallery.map((i) => i.toJson()).toList(),
    'duration': duration.inMilliseconds,
    'category' : category,
    'providers': providers,
    'rating1': rating1,
    'rating2': rating2,
    'rating3': rating3,
    'rating4': rating4,
    'rating5': rating5,
    'taxAdmin': taxAdmin,
    'addon': addon.map((i) => i.toJson()).toList(),
  };

  factory ServiceData.fromJson(String id, Map<String, dynamic> data){
    List<StringData> _name = [];
    if (data['name'] != null)
      List.from(data['name']).forEach((element){
        _name.add(StringData.fromJson(element));
      });
    List<StringData> _descTitle = [];
    if (data['descTitle'] != null)
      List.from(data['descTitle']).forEach((element){
        _descTitle.add(StringData.fromJson(element));
      });
    List<StringData> _desc = [];
    if (data['desc'] != null)
      List.from(data['desc']).forEach((element){
        _desc.add(StringData.fromJson(element));
      });
    List<PriceData> _price = [];
    if (data['price'] != null)
      List.from(data['price']).forEach((element){
        _price.add(PriceData.fromJson(element));
      });
    List<ImageData> _gallery = [];
    if (data['gallery'] != null)
      List.from(data['gallery']).forEach((element){
        _gallery.add(ImageData.fromJson(element));
      });
    List<String> _category = [];
    if (data['category'] != null)
      List.from(data['category']).forEach((element){
        _category.add(element);
      });
    List<String> _providers = [];
    if (data['providers'] != null)
      List.from(data['providers']).forEach((element){
        _providers.add(element);
      });
    var rating1 = (data["rating1"] != null) ? toInt(data["rating1"].toString()) : 0;
    var rating2 = (data["rating2"] != null) ? toInt(data["rating2"].toString()) : 0;
    var rating3 = (data["rating3"] != null) ? toInt(data["rating3"].toString()) : 0;
    var rating4 = (data["rating4"] != null) ? toInt(data["rating4"].toString()) : 0;
    var rating5 = (data["rating5"] != null) ? toInt(data["rating5"].toString()) : 0;
    var _count = rating1+rating2+rating3+rating4+rating5;
    double _rating = 0;
    if (_count != 0)
      _rating = (rating1*1 + rating2*2 + rating3*3 + rating4*4 + rating5*5)/_count;
    //
    List<AddonData> _addon = [];
    if (data['addon'] != null)
      List.from(data['addon']).forEach((element){
        _addon.add(AddonData.fromJson(element));
      });
    return ServiceData(
      id,
      _name,
      tax: (data["tax"] != null) ? toDouble(data["tax"].toString()) : 0,
      descTitle: _descTitle,
      desc: _desc,
      visible: (data["visible"] != null) ? data["visible"] : true,
      price: _price,
      gallery: _gallery,
      duration: (data["duration"] != null) ? Duration(milliseconds : data["duration"]) : Duration(),
      category: _category,
      providers: _providers,
      rating1: rating1,
      rating2: rating2,
      rating3: rating3,
      rating4: rating4,
      rating5: rating5,
      count: _count,
      rating: _rating,
      taxAdmin: (data["taxAdmin"] != null) ? toDouble(data["taxAdmin"].toString()) : 0,
      addon: _addon,
    );
  }

  //
  //
  //
  double getMinPrice(){
    double _min = double.maxFinite;
    for (var item in price) {
      if (item.discPrice == 0) {
        if (_min > item.price)
          _min = item.price;
      }else {
        if (_min > item.discPrice)
          _min = item.discPrice;
      }
    }
    if (_min == double.maxFinite)
      return 0;
    return _min;
  }
}


 */

/* customer v1
class ServiceData {
  String id;
  List<StringData> name;
  List<StringData> descTitle;
  List<StringData> desc;
  double tax;
  double taxAdmin;
  bool visible;
  List<PriceData> price;
  List<ImageData> gallery;
  Duration duration = Duration();
  List<String> category;
  List<String> providers; // Id
  //
  bool select = false;
  final dataKey = new GlobalKey();
  //
  int rating1;
  int rating2;
  int rating3;
  int rating4;
  int rating5;
  int count = 0;
  double rating = 0;
  //
  List<AddonData> addon;

  ServiceData(this.id, this.name, {this.visible = true, required this.desc,
    required this.gallery, required this.descTitle, required this.price,
    required this.duration, required this.category, required this.providers, this.tax = 0,
    this.rating1 = 0, this.rating2 = 0, this.rating3 = 0, this.rating4 = 0, this.rating5 = 0,
    this.count = 0, this.rating = 0, this.taxAdmin = 0, required this.addon
  });

  factory ServiceData.createEmpty(){
    return ServiceData("", [], gallery: [], price: [], desc: [], descTitle: [],
        duration: Duration(), category: [], providers: [], addon: []);
  }

  Map<String, dynamic> toJson() => {
    'name': name.map((i) => i.toJson()).toList(),
    'tax': tax,
    'descTitle': descTitle.map((i) => i.toJson()).toList(),
    'desc': desc.map((i) => i.toJson()).toList(),
    'visible': visible,
    'price': price.map((i) => i.toJson()).toList(),
    'gallery': gallery.map((i) => i.toJson()).toList(),
    'duration': duration.inMilliseconds,
    'category' : category,
    'providers': providers,
    'rating1': rating1,
    'rating2': rating2,
    'rating3': rating3,
    'rating4': rating4,
    'rating5': rating5,
    'taxAdmin': taxAdmin,
    'addon': addon.map((i) => i.toJson()).toList(),
  };

  factory ServiceData.fromJson(String id, Map<String, dynamic> data){
    List<StringData> _name = [];
    if (data['name'] != null)
      List.from(data['name']).forEach((element){
        _name.add(StringData.fromJson(element));
      });
    List<StringData> _descTitle = [];
    if (data['descTitle'] != null)
      List.from(data['descTitle']).forEach((element){
        _descTitle.add(StringData.fromJson(element));
      });
    List<StringData> _desc = [];
    if (data['desc'] != null)
      List.from(data['desc']).forEach((element){
        _desc.add(StringData.fromJson(element));
      });
    List<PriceData> _price = [];
    if (data['price'] != null)
      List.from(data['price']).forEach((element){
        _price.add(PriceData.fromJson(element));
      });
    List<ImageData> _gallery = [];
    if (data['gallery'] != null)
      List.from(data['gallery']).forEach((element){
        _gallery.add(ImageData.fromJson(element));
      });
    List<String> _category = [];
    if (data['category'] != null)
      List.from(data['category']).forEach((element){
        _category.add(element);
      });
    List<String> _providers = [];
    if (data['providers'] != null)
      List.from(data['providers']).forEach((element){
        _providers.add(element);
      });
    var rating1 = (data["rating1"] != null) ? toInt(data["rating1"].toString()) : 0;
    var rating2 = (data["rating2"] != null) ? toInt(data["rating2"].toString()) : 0;
    var rating3 = (data["rating3"] != null) ? toInt(data["rating3"].toString()) : 0;
    var rating4 = (data["rating4"] != null) ? toInt(data["rating4"].toString()) : 0;
    var rating5 = (data["rating5"] != null) ? toInt(data["rating5"].toString()) : 0;
    var _count = rating1+rating2+rating3+rating4+rating5;
    double _rating = 0;
    if (_count != 0)
      _rating = (rating1*1 + rating2*2 + rating3*3 + rating4*4 + rating5*5)/_count;
    //
    List<AddonData> _addon = [];
    if (data['addon'] != null)
      List.from(data['addon']).forEach((element){
        _addon.add(AddonData.fromJson(element));
      });
    return ServiceData(
      id,
      _name,
      tax: (data["tax"] != null) ? toDouble(data["tax"].toString()) : 0,
      descTitle: _descTitle,
      desc: _desc,
      visible: (data["visible"] != null) ? data["visible"] : true,
      price: _price,
      gallery: _gallery,
      duration: (data["duration"] != null) ? Duration(milliseconds : data["duration"]) : Duration(),
      category: _category,
      providers: _providers,
      rating1: rating1,
      rating2: rating2,
      rating3: rating3,
      rating4: rating4,
      rating5: rating5,
      count: _count,
      rating: _rating,
      taxAdmin: (data["taxAdmin"] != null) ? toDouble(data["taxAdmin"].toString()) : 0,
      addon: _addon,
    );
  }

  //
  //
  //
  double getMinPrice(){
    double _min = double.maxFinite;
    for (var item in price) {
      if (item.discPrice == 0) {
        if (_min > item.price)
          _min = item.price;
      }else {
        if (_min > item.discPrice)
          _min = item.discPrice;
      }
    }
    if (_min == double.maxFinite)
      return 0;
    return _min;
  }
}

 */