import 'dart:async';
import 'dart:io';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:flutter/material.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:uuid/uuid.dart';

import '../../abg_utils.dart';

List<ProviderData> providers = [];

Future<ProviderData?> getProviderData(String login) async {
  try{
    var querySnapshot = await FirebaseFirestore.instance.collection("provider").where("login", isEqualTo: login).get();
    for (var result in querySnapshot.docs) {
      var _data = result.data();
      if (_data.isNotEmpty){
        return ProviderData.fromJson(result.id, _data);
      }
    }
  }catch(ex){
    return null;
  }
  return null;
}

Future<String> getProviderId(String login) async {
  var ret = "";
  try{
    var querySnapshot = await FirebaseFirestore.instance.collection("provider").where("login", isEqualTo: login).get();
    for (var result in querySnapshot.docs) {
      var data = result.data();
      if (data.isEmpty)
        return "";
      ret = result.id;
    }
  }catch(ex){
    return "";
  }
  return ret;
}

ProviderData? getProviderById(String _providerId) {
  for (var item in providers)
    if (item.id == _providerId)
      return item;
}

String getProviderNameById(String _providerId, String locale) {
  ProviderData? provider = getProviderById(_providerId);
  if (provider == null)
    return "";
  return getTextByLocale(provider.name, locale);
}

ProviderData? getProviderByEmail(String userEmail) {
  for (var item in providers)
    if (item.login == userEmail)
      return item;
}

Future<String?> saveWorkArea(ProviderData _provider) async {
  try{
    await FirebaseFirestore.instance.collection("provider").doc(_provider.id).set({
      "route": _provider.route.map((i){
        return {'lat': i.latitude, 'lng': i.longitude};
      }).toList()
    }, SetOptions(merge: true));
  }catch(ex){
    return "saveWorkArea " + ex.toString();
  }
  return null;
}

String getProviderAddress(String id){
  String _address = "";
  for (var item in providers)
    if (item.id == id) {
      _address = item.address;
      break;
    }
  return _address;
}

Future<String?> setProviderAvailable(ProviderData provider, bool val) async {
  try{
    await FirebaseFirestore.instance.collection("provider").doc(provider.id).set({
      'available': val
    }, SetOptions(merge: true));
  }catch(ex){
    return "setProviderAvailable " + ex.toString();
  }
  return null;
}

Future<String?> registerProvider(String email, String pass, String _login,
    String providerName, String phone, String desc, String address,
    List<String> category, String pathToImage, List<LatLng> routeForNewProvider) async {
  try {
    final User? user = (await FirebaseAuth.instance.createUserWithEmailAndPassword(
      email: email, password: pass,)).user;
    if (user == null)
      return "User don't create";

    // load image
    var f = Uuid().v4();
    var name = "provider/$f.jpg";
    var firebaseStorageRef = FirebaseStorage.instance.ref().child(name);
    TaskSnapshot s = await firebaseStorageRef.putFile(File(pathToImage));

    await FirebaseFirestore.instance.collection("listusers").doc(user.uid).set({
      "visible": true,
      "providerApp": true,
      "providerRequest" : true,
      "phoneVerified": false,
      "email": user.email,
      "phone": phone,
      "name": _login,
      "date_create" : FieldValue.serverTimestamp(),
      "providerName": providerName,
      "providerAddress": address,
      "providerDesc": desc,
      "providerLogoLocalFile": name,
      "providerLogoServerPath": await s.ref.getDownloadURL(),
      "providerWorkArea": routeForNewProvider.map((i){
        return {'lat': i.latitude, 'lng': i.longitude};
      }).toList(),
      "providerCategory": category
    }, SetOptions(merge:true));

    await FirebaseFirestore.instance.collection("settings").doc("main")
        .set({"provider_request_count": FieldValue.increment(1)}, SetOptions(merge:true));
    await FirebaseFirestore.instance.collection("settings").doc("main")
        .set({"provider_new_request_count": FieldValue.increment(1)}, SetOptions(merge:true));

  }catch(ex){
    return "model registerProvider " + ex.toString();
  }
  return null;
}

Future<String?> loadProvider() async{
  try{
    var querySnapshot = await FirebaseFirestore.instance.collection("provider").get();
    providers = [];
    for (var result in querySnapshot.docs) {
      var _data = result.data();
      dprint("Provider $_data");
      var t = ProviderData.fromJson(result.id, _data);
      providers.add(t);
    }
    //parent.stat.addStat("provider", parent.providers.length);
  }catch(ex){
    return "loadProvider " + ex.toString();
  }
  return null;
}

class ProviderData {
  String id;
  List<StringData> name;
  String phone;
  String www;
  String instagram;
  String telegram;
  List<StringData> desc;
  List<StringData> descTitle;
  String address;
  //String avatar;
  bool visible;
  int unread = 0;
  int all = 0;
  String login;
  String lastMessage = "";
  DateTime lastMessageTime = DateTime.now();
  String chatId = "";

  String imageUpperServerPath = "";
  String imageUpperLocalFile = "";
  String logoServerPath = "";
  String logoLocalFile = "";
  List<ImageData> gallery = [];
  //
  List<WorkTimeData> workTime = [];
  List<String> category = [];
  //
  bool select = false;
  final dataKey = GlobalKey();
  List<LatLng> route;
  double distanceToUser = 0;
  bool available = true;
  String assetUpperImage;
  String assetsLogo;
  List<String> assetsGallery;
  List<String> assetsCategory;
  List<AddonData> addon;

  ProviderData({this.id = "", this.name = const [], this.visible = true, this.address = "", this.desc = const [],
    this.phone = "", this.www = "", this.instagram = "", this.telegram = "", this.descTitle = const [],
    this.imageUpperServerPath = "", this.imageUpperLocalFile = "",
    this.logoServerPath = "", this.logoLocalFile = "", this.gallery = const [], this.workTime = const [],
    this.category = const [], //this.avatar = ""
    this.login = "", required this.route, this.available = true,
    this.assetUpperImage = "", this.assetsLogo = "", this.assetsGallery = const [], this.assetsCategory = const [],
    required this.addon,
  });

  factory ProviderData.createEmpty(){
    return ProviderData(descTitle: [StringData(code: "en", text: "Description",)], route: [], addon: []);
  }

  Map<String, dynamic> toJson() => {
    'name': name.map((i) => i.toJson()).toList(),
    'phone': phone,
    'www': www,
    'instagram': instagram,
    'telegram': telegram,
    'desc': desc.map((i) => i.toJson()).toList(),
    'descTitle': descTitle.map((i) => i.toJson()).toList(),
    'address': address,
    'visible': visible,
    'imageUpperServerPath': imageUpperServerPath,
    'imageUpperLocalFile': imageUpperLocalFile,
    'logoServerPath': logoServerPath,
    'logoLocalFile': logoLocalFile,
    'gallery': gallery.map((i) => i.toJson()).toList(),
    'workTime': workTime.map((i) => i.toJson()).toList(),
    'category': category,
    "login" : login,
    "route" : route.map((i){
      return {'lat': i.latitude, 'lng': i.longitude};
    }).toList(),
    'available': available,
    'addon': addon.map((i) => i.toJson()).toList(),
  };

  factory ProviderData.fromJson(String id, Map<String, dynamic> data){
    List<String> _category = [];
    if (data['category'] != null)
      for (var element in List.from(data['category'])) {
        _category.add(element);
      }
    List<ImageData> _gallery = [];
    if (data['gallery'] != null)
      for (var element in List.from(data['gallery'])) {
        _gallery.add(ImageData(serverPath: element["serverPath"], localFile: element["localFile"]));
      }
    //
    List<WorkTimeData> _workTime = [];
    if (data['workTime'] != null)
      for (var element in List.from(data['workTime'])) {
        _workTime.add(WorkTimeData.fromJson(element));
      }
    //
    List<StringData> _name = [];
    if (data['name'] != null)
      for (var element in List.from(data['name'])) {
        _name.add(StringData.fromJson(element));
      }
    List<StringData> _desc = [];
    if (data['desc'] != null)
      for (var element in List.from(data['desc'])) {
        _desc.add(StringData.fromJson(element));
      }
    List<StringData> _descTitle = [];
    if (data['descTitle'] != null)
      for (var element in List.from(data['descTitle'])) {
        _descTitle.add(StringData.fromJson(element));
      }
    List<LatLng> _route = [];
    if (data["route"] != null)
      for (var element in List.from(data['route'])) {
        if (element['lat'] != null && element['lng'] != null)
          _route.add(LatLng(
              element['lat'], element['lng']
          ));
      }
    List<AddonData> _addon = [];
    if (data['addon'] != null)
      for (var element in List.from(data['addon'])) {
        _addon.add(AddonData.fromJson(element));
      }
    return ProviderData(
      id: id,
      name: _name,
      phone: (data["phone"] != null) ? data["phone"] : "",
      www: (data["www"] != null) ? data["www"] : "",
      instagram: (data["instagram"] != null) ? data["instagram"] : "",
      telegram: (data["telegram"] != null) ? data["telegram"] : "",
      desc: _desc,
      descTitle: _descTitle,
      address: (data["address"] != null) ? data["address"] : "",
      visible: (data["visible"] != null) ? data["visible"] : true,
      imageUpperServerPath: (data["imageUpperServerPath"] != null) ? data["imageUpperServerPath"] : "",
      imageUpperLocalFile: (data["imageUpperLocalFile"] != null) ? data["imageUpperLocalFile"] : "",
      logoServerPath: (data["logoServerPath"] != null) ? data["logoServerPath"] : "",
      logoLocalFile: (data["logoLocalFile"] != null) ? data["logoLocalFile"] : "",
      gallery: _gallery,
      workTime: _workTime,
      category: _category,
      //avatar: (data["avatar"] != null) ? data["avatar"] : "",
      login: (data["login"] != null) ? data["login"] : "",
      route : _route,
      available: (data["available"] != null) ? data["available"] : true,
      addon: _addon,
    );
  }
}

class WorkTimeData{
  int id = 0;
  bool weekend = false;
  String openTime = "";
  String closeTime = "";

  WorkTimeData({this.id = 0, this.openTime = "09:00", this.closeTime = "16:00", this.weekend = false});

  Map<String, dynamic> toJson() => {
    'index': id,
    'openTime': openTime,
    'closeTime': closeTime,
    'weekend': weekend,
  };

  factory WorkTimeData.fromJson(Map<String, dynamic> data){
    return WorkTimeData(
      id: (data["index"] != null) ? data["index"] : 0,
      openTime: (data["openTime"] != null) ? data["openTime"] : "9:00",
      closeTime: (data["closeTime"] != null) ? data["closeTime"] : "16:00",
      weekend: (data["weekend"] != null) ? data["weekend"] : false,
    );
  }

  factory WorkTimeData.createEmpty(){
    return WorkTimeData();
  }
}


/* provider v2

class ProviderData {
  String id;
  List<StringData> name;
  String phone;
  String www;
  String instagram;
  String telegram;
  List<StringData> desc;
  List<StringData> descTitle;
  String address;
  //String avatar;
  bool visible;
  int unread = 0;
  int all = 0;
  String login;
  String lastMessage = "";
  DateTime lastMessageTime = DateTime.now();
  String chatId = "";

  String imageUpperServerPath = "";
  String imageUpperLocalFile = "";
  String logoServerPath = "";
  String logoLocalFile = "";
  List<ImageData> gallery = [];
  //
  List<WorkTimeData> workTime = [];
  List<String> category = [];
  //
  List<AddonData> addon;
  List<gm.LatLng> route;
  //
  bool select = false;
  final dataKey = new GlobalKey();

  ProviderData({this.id = "", this.name = const [], this.visible = true, this.address = "", this.desc = const [],
    this.phone = "", this.www = "", this.instagram = "", this.telegram = "", this.descTitle = const [],
    this.imageUpperServerPath = "", this.imageUpperLocalFile = "",
    this.logoServerPath = "", this.logoLocalFile = "", this.gallery = const [], this.workTime = const [],
    this.category = const [], //this.avatar = ""
    this.login = "", required this.addon, required this.route,
  });

  factory ProviderData.createEmpty(){
    return ProviderData(descTitle: [StringData(code: "en", text: strings.get(122))], addon: [], route: [],); /// "Description",
  }

  Map<String, dynamic> toJson() => {
    'name': name.map((i) => i.toJson()).toList(),
    'phone': phone,
    'www': www,
    'instagram': instagram,
    'telegram': telegram,
    'desc': desc.map((i) => i.toJson()).toList(),
    'descTitle': descTitle.map((i) => i.toJson()).toList(),
    'address': address,
    'visible': visible,
    'imageUpperServerPath': imageUpperServerPath,
    'imageUpperLocalFile': imageUpperLocalFile,
    'logoServerPath': logoServerPath,
    'logoLocalFile': logoLocalFile,
    'gallery': gallery.map((i) => i.toJson()).toList(),
    'workTime': workTime.map((i) => i.toJson()).toList(),
    'category': category,
    "login" : login,
    "route" : route.map((i){
      return {'lat': i.latitude, 'lng': i.longitude};
    }).toList(),
    'addon': addon.map((i) => i.toJson()).toList(),
  };

  factory ProviderData.fromJson(String id, Map<String, dynamic> data){
    List<String> _category = [];
    if (data['category'] != null)
      List.from(data['category']).forEach((element){
        _category.add(element);
      });
    List<ImageData> _gallery = [];
    if (data['gallery'] != null)
      List.from(data['gallery']).forEach((element){
        _gallery.add(ImageData(serverPath: element["serverPath"], localFile: element["localFile"]));
      });
    //
    List<WorkTimeData> _workTime = [];
    if (data['workTime'] != null)
      List.from(data['workTime']).forEach((element){
        _workTime.add(WorkTimeData.fromJson(element));
      });
    //
    List<StringData> _name = [];
    if (data['name'] != null)
      List.from(data['name']).forEach((element){
        _name.add(StringData.fromJson(element));
      });
    List<StringData> _desc = [];
    if (data['desc'] != null)
      List.from(data['desc']).forEach((element){
        _desc.add(StringData.fromJson(element));
      });
    List<StringData> _descTitle = [];
    if (data['descTitle'] != null)
      List.from(data['descTitle']).forEach((element){
        _descTitle.add(StringData.fromJson(element));
      });
    List<AddonData> _addon = [];
    if (data['addon'] != null)
      List.from(data['addon']).forEach((element){
        _addon.add(AddonData.fromJson(element));
      });
    List<gm.LatLng> _route = [];
    if (data["route"] != null)
      List.from(data['route']).forEach((element){
        if (element['lat'] != null && element['lng'] != null)
          _route.add(gm.LatLng(
              element['lat'], element['lng']
          ));
      });
    return ProviderData(
      id: id,
      name: _name,
      phone: (data["phone"] != null) ? data["phone"] : "",
      www: (data["www"] != null) ? data["www"] : "",
      instagram: (data["instagram"] != null) ? data["instagram"] : "",
      telegram: (data["telegram"] != null) ? data["telegram"] : "",
      desc: _desc,
      descTitle: _descTitle,
      address: (data["address"] != null) ? data["address"] : "",
      visible: (data["visible"] != null) ? data["visible"] : true,
      imageUpperServerPath: (data["imageUpperServerPath"] != null) ? data["imageUpperServerPath"] : "",
      imageUpperLocalFile: (data["imageUpperLocalFile"] != null) ? data["imageUpperLocalFile"] : "",
      logoServerPath: (data["logoServerPath"] != null) ? data["logoServerPath"] : "",
      logoLocalFile: (data["logoLocalFile"] != null) ? data["logoLocalFile"] : "",
      gallery: _gallery,
      workTime: _workTime,
      category: _category,
      //avatar: (data["avatar"] != null) ? data["avatar"] : "",
      login: (data["login"] != null) ? data["login"] : "",
      route : _route,
      addon: _addon,
    );
  }
}


class WorkTimeData{
  int id = 0;
  bool weekend = false;
  String openTime = "";
  String closeTime = "";

  WorkTimeData({this.id = 0, this.openTime = "09:00", this.closeTime = "16:00", this.weekend = false});

  Map<String, dynamic> toJson() => {
    'index': id,
    'openTime': openTime,
    'closeTime': closeTime,
    'weekend': weekend,
  };

  factory WorkTimeData.fromJson(Map<String, dynamic> data){
    return WorkTimeData(
      id: (data["index"] != null) ? data["index"] : 0,
      openTime: (data["openTime"] != null) ? data["openTime"] : "9:00",
      closeTime: (data["closeTime"] != null) ? data["closeTime"] : "16:00",
      weekend: (data["weekend"] != null) ? data["weekend"] : false,
    );
  }

  factory WorkTimeData.createEmpty(){
    return WorkTimeData();
  }
}
 */