import 'package:cloud_firestore/cloud_firestore.dart';

import '../../abg_utils.dart';

int reviewsRate = 0;
String reviewsRateString = "";

List<ReviewsData> reviews = [];

Future<String?> loadReviewsForProviderScreen(String providerId) async{
  try{
    var querySnapshot = await FirebaseFirestore.instance.collection("reviews")
        .where("provider", isEqualTo: providerId).get(); //  parent.currentProvider.id
    reviews = [];
    reviewsRate = 0;
    reviewsRateString = "";
    for (var result in querySnapshot.docs) {
      var _data = result.data();
      // dprint("loadReviews $_data");
      reviews.add(ReviewsData.fromJson(result.id, _data));
    }
    if (reviews.isNotEmpty)
      _calculateReview();
    reviews.sort((a, b) => b.time.compareTo(a.time));
  }catch(ex){
    return "model loadReviews " + ex.toString();
  }
  return null;
}

_calculateReview(){
  double rating = 0;
  for (var item in reviews)
    rating += item.rating;
  reviewsRate = rating~/reviews.length;
  reviewsRateString = (rating/reviews.length).toStringAsFixed(1);
}

class ReviewsData{
  final String id;
  final int rating;
  final String text;
  List<ImageData> images = [];
  final String user;
  final String serviceId;
  final String serviceName;
  final String providerId;
  final String userName;
  final String userAvatar;
  final DateTime time;

  ReviewsData({this.id = "", this.rating = 0, this.text = "", required this.images, this.user = "",
    this.serviceId = "", this.serviceName = "", this.providerId = "", this.userName = "",
    this.userAvatar = "", required this.time});

  Map<String, dynamic> toJson() => {
    "rating": rating,
    "text": text,
    "images": images.map((i) => i.toJson()).toList(),
    "user": user,
    "service": serviceId,
    'serviceName': serviceName,
    "provider": providerId,
    "userName": userName,
    "userAvatar": userAvatar,
    "time" : time
  };

  factory ReviewsData.fromJson(String _id, Map<String, dynamic> data){
    List<ImageData> _images = [];
    if (data['images'] != null)
      for (var element in List.from(data['images'])) {
        _images.add(ImageData(serverPath: element["serverPath"], localFile: element["localFile"]));
      }
    return ReviewsData(
      id : _id,
      rating: (data["rating"] != null) ? toInt(data["rating"].toString()) : 0,
      text: (data["text"] != null) ? data["text"] : "",
      images: _images,
      user: (data["user"] != null) ? data["user"] : "",
      serviceId: (data["service"] != null) ? data["service"] : "",
      serviceName: (data["serviceName"] != null) ? data["serviceName"] : "",
      providerId: (data["provider"] != null) ? data["provider"] : "",
      userName: (data["userName"] != null) ? data["userName"] : "",
      userAvatar: (data["userAvatar"] != null) ? data["userAvatar"] : "",
      time: (data["time"] != null) ? data["time"].toDate().toLocal() : DateTime.now(),
    );
  }
}

